@@ -572,22 +572,22 @@ compare_avc_ref_store_func(const void *p1, const void *p2)
     return fs1->ref_age - fs2->ref_age;
 }
 
-void
-intel_update_avc_frame_store_index(
+static void
+intel_update_codec_frame_store_index(
     VADriverContextP              ctx,
     struct decode_state          *decode_state,
-    VAPictureParameterBufferH264 *pic_param,
-    GenFrameStore                 frame_store[MAX_GEN_REFERENCE_FRAMES],
+    int poc,
+    GenFrameStore                 frame_store[],
+    int num_elements,
     GenFrameStoreContext         *fs_ctx
 )
 {
-    GenFrameStore *free_refs[MAX_GEN_REFERENCE_FRAMES];
+    GenFrameStore **free_refs = calloc(num_elements, sizeof(GenFrameStore *));
     uint32_t used_refs = 0, add_refs = 0;
     uint64_t age;
     int i, n, num_free_refs;
 
     /* Detect changes of access unit */
-    const int poc = avc_get_picture_poc(&pic_param->CurrPic);
     if (fs_ctx->age == 0 || fs_ctx->prev_poc != poc)
         fs_ctx->age++;
     fs_ctx->prev_poc = poc;
@@ -600,12 +600,12 @@ intel_update_avc_frame_store_index(
         if (!obj_surface)
             continue;
 
-        GenAvcSurface * const avc_surface = obj_surface->private_data;
-        if (!avc_surface)
+        GenCodecSurface * const codec_surface = obj_surface->private_data;
+        if (!codec_surface)
             continue;
-        if (avc_surface->frame_store_id >= 0) {
+        if (codec_surface->frame_store_id >= 0) {
             GenFrameStore * const fs =
-                &frame_store[avc_surface->frame_store_id];
+                &frame_store[codec_surface->frame_store_id];
             if (fs->surface_id == obj_surface->base.id) {
                 fs->obj_surface = obj_surface;
                 fs->ref_age = age;
@@ -618,7 +618,7 @@ intel_update_avc_frame_store_index(
 
     /* Build and sort out the list of retired candidates. The resulting
        list is ordered by increasing age when they were last used */
-    for (i = 0, n = 0; i < MAX_GEN_REFERENCE_FRAMES; i++) {
+    for (i = 0, n = 0; i < num_elements; i++) {
         if (!(used_refs & (1 << i))) {
             GenFrameStore * const fs = &frame_store[i];
             fs->obj_surface = NULL;
@@ -635,8 +635,8 @@ intel_update_avc_frame_store_index(
         if (!obj_surface || !(add_refs & (1 << i)))
             continue;
 
-        GenAvcSurface * const avc_surface = obj_surface->private_data;
-        if (!avc_surface)
+        GenCodecSurface * const codec_surface = obj_surface->private_data;
+        if (!codec_surface)
             continue;
         if (n < num_free_refs) {
             GenFrameStore * const fs = free_refs[n++];
@@ -644,11 +644,47 @@ intel_update_avc_frame_store_index(
             fs->obj_surface = obj_surface;
             fs->frame_store_id = fs - frame_store;
             fs->ref_age = age;
-            avc_surface->frame_store_id = fs->frame_store_id;
+            codec_surface->frame_store_id = fs->frame_store_id;
             continue;
         }
         WARN_ONCE("No free slot found for DPB reference list!!!\n");
     }
+
+    free(free_refs);
+}
+
+void
+intel_update_avc_frame_store_index(
+    VADriverContextP              ctx,
+    struct decode_state          *decode_state,
+    VAPictureParameterBufferH264 *pic_param,
+    GenFrameStore                 frame_store[MAX_GEN_REFERENCE_FRAMES],
+    GenFrameStoreContext         *fs_ctx
+)
+{
+    intel_update_codec_frame_store_index(ctx,
+                                         decode_state,
+                                         avc_get_picture_poc(&pic_param->CurrPic),
+                                         frame_store,
+                                         MAX_GEN_REFERENCE_FRAMES,
+                                         fs_ctx);
+}
+
+void
+intel_update_hevc_frame_store_index(
+    VADriverContextP              ctx,
+    struct decode_state          *decode_state,
+    VAPictureParameterBufferHEVC *pic_param,
+    GenFrameStore                 frame_store[MAX_GEN_HCP_REFERENCE_FRAMES],
+    GenFrameStoreContext         *fs_ctx
+    )
+{
+    intel_update_codec_frame_store_index(ctx,
+                                         decode_state,
+                                         pic_param->CurrPic.pic_order_cnt,
+                                         frame_store,
+                                         MAX_GEN_HCP_REFERENCE_FRAMES,
+                                         fs_ctx);
 }
 
 void
@@ -1055,6 +1091,89 @@ intel_decoder_check_vp8_parameter(VADriverContextP ctx,
     return VA_STATUS_SUCCESS;
 }
 
+VAStatus
+hevc_ensure_surface_bo(
+    VADriverContextP                    ctx,
+    struct decode_state                *decode_state,
+    struct object_surface              *obj_surface,
+    const VAPictureParameterBufferHEVC *pic_param
+)
+{
+    VAStatus va_status = VA_STATUS_SUCCESS;
+
+    /* (Re-)allocate the underlying surface buffer store, if necessary */
+    if (!obj_surface->bo || obj_surface->fourcc != VA_FOURCC_NV12) {
+        struct i965_driver_data * const i965 = i965_driver_data(ctx);
+
+        i965_destroy_surface_storage(obj_surface);
+
+        va_status = i965_check_alloc_surface_bo(ctx,
+                                                obj_surface,
+                                                i965->codec_info->has_tiled_surface,
+                                                VA_FOURCC_NV12,
+                                                SUBSAMPLE_YUV420);
+    }
+
+    return va_status;
+}
+
+static VAStatus
+intel_decoder_check_hevc_parameter(VADriverContextP ctx,
+                                   struct decode_state *decode_state)
+{
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    VAPictureParameterBufferHEVC *pic_param = (VAPictureParameterBufferHEVC *)decode_state->pic_param->buffer;
+    VAStatus va_status = VA_STATUS_ERROR_INVALID_PARAMETER;
+    struct object_surface *obj_surface;
+    int i;
+    int min_cb_size;
+
+    if (pic_param->CurrPic.flags & VA_PICTURE_HEVC_INVALID ||
+        pic_param->CurrPic.picture_id == VA_INVALID_SURFACE)
+        goto error;
+
+    if (pic_param->CurrPic.picture_id != decode_state->current_render_target)
+        goto error;
+
+    min_cb_size = (1 << (pic_param->log2_min_luma_coding_block_size_minus3 + 3));
+
+    if (pic_param->pic_width_in_luma_samples % min_cb_size ||
+        pic_param->pic_height_in_luma_samples % min_cb_size)
+        goto error;
+
+    /* Fill in the reference objects array with the actual VA surface
+       objects with 1:1 correspondance with any entry in ReferenceFrames[],
+       i.e. including "holes" for invalid entries, that are expanded
+       to NULL in the reference_objects[] array */
+    for (i = 0; i < ARRAY_ELEMS(pic_param->ReferenceFrames); i++) {
+        const VAPictureHEVC * const va_pic = &pic_param->ReferenceFrames[i];
+
+        obj_surface = NULL;
+        if (!(va_pic->flags & VA_PICTURE_HEVC_INVALID) &&
+            va_pic->picture_id != VA_INVALID_ID) {
+            obj_surface = SURFACE(pic_param->ReferenceFrames[i].picture_id);
+
+            if (!obj_surface) {
+                va_status = VA_STATUS_ERROR_INVALID_SURFACE;
+                goto error;
+            }
+
+            va_status = hevc_ensure_surface_bo(ctx, decode_state, obj_surface,
+                                               pic_param);
+
+            if (va_status != VA_STATUS_SUCCESS)
+                goto error;
+        }
+
+        decode_state->reference_objects[i] = obj_surface;
+    }
+
+    va_status = VA_STATUS_SUCCESS;
+
+error:
+    return va_status;
+}
+
 VAStatus
 intel_decoder_sanity_check_input(VADriverContextP ctx,
                                  VAProfile profile,
@@ -1102,6 +1221,11 @@ intel_decoder_sanity_check_input(VADriverContextP ctx,
         vaStatus = intel_decoder_check_vp8_parameter(ctx, decode_state);
         break;
 
+    case VAProfileHEVCMain:
+    case VAProfileHEVCMain10:
+        vaStatus = intel_decoder_check_hevc_parameter(ctx, decode_state);
+        break;
+
     default:
         vaStatus = VA_STATUS_ERROR_INVALID_PARAMETER;
         break;
@@ -1175,3 +1299,25 @@ intel_ensure_vp8_segmentation_buffer(VADriverContextP ctx, GenBuffer *buf,
     buf->valid = buf->bo != NULL;
     return buf->valid;
 }
+
+void
+hevc_gen_default_iq_matrix(VAIQMatrixBufferHEVC *iq_matrix)
+{
+    /* Flat_4x4_16 */
+    memset(&iq_matrix->ScalingList4x4, 16, sizeof(iq_matrix->ScalingList4x4));
+
+    /* Flat_8x8_16 */
+    memset(&iq_matrix->ScalingList8x8, 16, sizeof(iq_matrix->ScalingList8x8));
+
+    /* Flat_16x16_16 */
+    memset(&iq_matrix->ScalingList16x16, 16, sizeof(iq_matrix->ScalingList16x16));
+
+    /* Flat_32x32_16 */
+    memset(&iq_matrix->ScalingList32x32, 16, sizeof(iq_matrix->ScalingList32x32));
+
+    /* Flat_16x16_dc_16 */
+    memset(&iq_matrix->ScalingListDC16x16, 16, sizeof(iq_matrix->ScalingListDC16x16));
+
+    /* Flat_32x32_dc_16 */
+    memset(&iq_matrix->ScalingListDC32x32, 16, sizeof(iq_matrix->ScalingListDC32x32));
+}
