@@ -327,6 +327,9 @@
 #define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_HS          CMD(3, 0, 0x2c)
 #define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_DS          CMD(3, 0, 0x2d)
 
+#define GEN7_3DSTATE_VF                                 CMD(3, 0, 0x0c)
+#define GEN8_3DSTATE_VF_INSTANCING                      CMD(3, 0, 0x49)
+
 #define MFX(pipeline, op, sub_opa, sub_opb)     \
     (3 << 29 |                                  \
      (pipeline) << 27 |                         \
@@ -379,9 +382,14 @@
 
 #define MFD_VC1_BSD_OBJECT                      MFX(2, 2, 1, 8)
 
+//MFX_JPEG #defines for JPEG decoder only
 #define MFX_JPEG_PIC_STATE                      MFX(2, 7, 0, 0)
 #define MFX_JPEG_HUFF_TABLE_STATE               MFX(2, 7, 0, 2)
 
+//MFC_JPEG #defines for JPEG encoder only
+#define MFC_JPEG_SCAN_OBJECT                    MFX(2, 7, 2, 9)
+#define MFC_JPEG_HUFF_TABLE_STATE               MFX(2, 7, 2, 3)
+
 #define MFD_JPEG_BSD_OBJECT                     MFX(2, 7, 1, 8)
 
 #define MFX_VP8_PIC_STATE                       MFX(2, 4, 0, 0)
@@ -400,6 +408,27 @@
 #define VEB_STATE                               VEB(2, 4, 0, 2)
 #define VEB_DNDI_IECP_STATE                     VEB(2, 4, 0, 3)
 
+#define HCP(command)                            \
+    (3 << 29 |                                  \
+     2 << 27 |                                  \
+     7 << 23 |                                  \
+     (command << 16))
+
+#define HCP_PIPE_MODE_SELECT                    HCP(0x00)
+#define HCP_SURFACE_STATE                       HCP(0x01)
+#define HCP_PIPE_BUF_ADDR_STATE                 HCP(0x02)
+#define HCP_IND_OBJ_BASE_ADDR_STATE             HCP(0x03)
+#define HCP_QM_STATE                            HCP(0x04)
+#define HCP_FQM_STATE                           HCP(0x05)
+#define HCP_PIC_STATE                           HCP(0x10)
+#define HCP_TILE_STATE                          HCP(0x11)
+#define HCP_REF_IDX_STATE                       HCP(0x12)
+#define HCP_WEIGHTOFFSET                        HCP(0x13)
+#define HCP_SLICE_STATE                         HCP(0x14)
+#define HCP_BSD_OBJECT                          HCP(0x20)
+#define HCP_PAK_OBJECT                          HCP(0x21)
+#define HCP_INSERT_PAK_OBJECT                   HCP(0x22)
+
 #define I965_DEPTHFORMAT_D32_FLOAT              1
 
 #define BASE_ADDRESS_MODIFY             (1 << 0)
@@ -800,9 +829,12 @@
 #define MFD_MODE_VLD            0
 #define MFD_MODE_IT             1
 
+#define MFX_SURFACE_YCRCB_NORMAL        0
+#define MFX_SURFACE_YCRCB_SWAPY         3
 #define MFX_SURFACE_PLANAR_420_8        4
 #define MFX_SURFACE_PLANAR_411_8        5
 #define MFX_SURFACE_PLANAR_422_8        6
+#define MFX_SURFACE_R8G8B8A8_UNORM      9
 #define MFX_SURFACE_MONOCHROME          12
 
 #define MPEG_I_PICTURE          1
@@ -820,5 +852,49 @@
 #define SUBSAMPLE_YUV444        4
 #define SUBSAMPLE_YUV411        5
 #define SUBSAMPLE_RGBX          6
+#define JPEG_ENC_SURFACE_NV12 1
+#define JPEG_ENC_SURFACE_UYVY 2
+#define JPEG_ENC_SURFACE_YUY2 3
+#define JPEG_ENC_SURFACE_Y8   4
+#define JPEG_ENC_SURFACE_RGB  5
+
+#define JPEG_ENC_MCU_YUV400     0
+#define JPEG_ENC_MCU_YUV420     1
+#define JPEG_ENC_MCU_YUV422H_2Y 2
+#define JPEG_ENC_MCU_RGB        3
+
+#define JPEG_ENC_ROUND_QUANT_DEFAULT 0
+#define JPEG_ENC_ROUND_QUANT_MINUS_128TH 1
+#define JPEG_ENC_ROUND_QUANT_PLUS_128TH 2
+#define JPEG_ENC_ROUND_QUANT_MINUS_64TH 3
+#define JPEG_ENC_ROUND_QUANT_PLUS_64TH 4
+#define JPEG_ENC_ROUND_QUANT_MINUS_32TH 5
+#define JPEG_ENC_ROUND_QUANT_MINUS_16TH 6
+#define JPEG_ENC_ROUND_QUANT_MINUS_8TH 7
+
+
+
+#define GEN9_PIPELINE_SELECTION_MASK        (0x3 << 8)
+#define GEN9_FORCE_MEDIA_AWAKE_MASK         (1 << 13)
+#define GEN9_FORCE_MEDIA_AWAKE_ON           (1 << 5)
+#define GEN9_FORCE_MEDIA_AWAKE_OFF          (0 << 5)
+
+#define GEN9_MEDIA_DOP_GATE_MASK            (1 << 12)
+#define GEN9_MEDIA_DOP_GATE_ON              (1 << 4)
+#define GEN9_MEDIA_DOP_GATE_OFF             (0 << 4)
+
+#define GEN9_SBE_ACTIVE_COMPONENT_NONE          0
+#define GEN9_SBE_ACTIVE_COMPONENT_XY            1
+#define GEN9_SBE_ACTIVE_COMPONENT_XYZ           2
+#define GEN9_SBE_ACTIVE_COMPONENT_XYZW          3
+
+#define HEVC_SLICE_B                            0
+#define HEVC_SLICE_P                            1
+#define HEVC_SLICE_I                            2
+
+#define HCP_CODEC_HEVC                          0
+
+#define HCP_CODEC_SELECT_DECODE                 0
+#define HCP_CODEC_SELECT_ENCODE                 1
 
 #endif /* _I965_DEFINES_H_ */
