@@ -41,6 +41,10 @@
     (VA_RT_FORMAT_YUV400 | VA_RT_FORMAT_YUV411 | VA_RT_FORMAT_YUV422 | \
      VA_RT_FORMAT_YUV444)
 
+/* Extra set of chroma formats supported for JPEG encoding (beyond YUV 4:2:0) */
+#define EXTRA_JPEG_ENC_CHROMA_FORMATS \
+    (VA_RT_FORMAT_YUV400| VA_RT_FORMAT_YUV422 | VA_RT_FORMAT_YUV444 | VA_RT_FORMAT_RGB32)
+     
 /* Defines VA profile as a 32-bit unsigned integer mask */
 #define VA_PROFILE_MASK(PROFILE) \
     (1U << VAProfile##PROFILE)
@@ -89,6 +93,8 @@ static struct hw_codec_info ilk_hw_codec_info = {
     .num_filters = 0,
 };
 
+static void gen6_hw_codec_preinit(VADriverContextP ctx, struct hw_codec_info *codec_info);
+
 extern struct hw_context *gen6_dec_hw_context_init(VADriverContextP, struct object_config *);
 extern struct hw_context *gen6_enc_hw_context_init(VADriverContextP, struct object_config *);
 static struct hw_codec_info snb_hw_codec_info = {
@@ -97,6 +103,7 @@ static struct hw_codec_info snb_hw_codec_info = {
     .proc_hw_context_init = i965_proc_context_init,
     .render_init = genx_render_init,
     .post_processing_context_init = i965_post_processing_context_init,
+    .preinit_hw_codec = gen6_hw_codec_preinit,
 
     .max_width = 2048,
     .max_height = 2048,
@@ -123,6 +130,8 @@ static struct hw_codec_info snb_hw_codec_info = {
     },
 };
 
+static void gen7_hw_codec_preinit(VADriverContextP ctx, struct hw_codec_info *codec_info);
+
 extern struct hw_context *gen7_dec_hw_context_init(VADriverContextP, struct object_config *);
 extern struct hw_context *gen7_enc_hw_context_init(VADriverContextP, struct object_config *);
 static struct hw_codec_info ivb_hw_codec_info = {
@@ -131,6 +140,7 @@ static struct hw_codec_info ivb_hw_codec_info = {
     .proc_hw_context_init = i965_proc_context_init,
     .render_init = genx_render_init,
     .post_processing_context_init = i965_post_processing_context_init,
+    .preinit_hw_codec = gen7_hw_codec_preinit,
 
     .max_width = 4096,
     .max_height = 4096,
@@ -268,6 +278,7 @@ static struct hw_codec_info chv_hw_codec_info = {
                               VA_PROFILE_MASK(H264MultiviewHigh)),
     .h264_dec_chroma_formats = EXTRA_H264_DEC_CHROMA_FORMATS,
     .jpeg_dec_chroma_formats = EXTRA_JPEG_DEC_CHROMA_FORMATS,
+    .jpeg_enc_chroma_formats = EXTRA_JPEG_ENC_CHROMA_FORMATS,
 
     .has_mpeg2_decoding = 1,
     .has_mpeg2_encoding = 1,
@@ -275,6 +286,7 @@ static struct hw_codec_info chv_hw_codec_info = {
     .has_h264_encoding = 1,
     .has_vc1_decoding = 1,
     .has_jpeg_decoding = 1,
+    .has_jpeg_encoding = 1,
     .has_vpp = 1,
     .has_accelerated_getimage = 1,
     .has_accelerated_putimage = 1,
@@ -294,6 +306,54 @@ static struct hw_codec_info chv_hw_codec_info = {
     },
 };
 
+extern struct hw_context *gen9_dec_hw_context_init(VADriverContextP, struct object_config *);
+extern struct hw_context *gen9_enc_hw_context_init(VADriverContextP, struct object_config *);
+extern void gen9_post_processing_context_init(VADriverContextP, void *, struct intel_batchbuffer *);
+static struct hw_codec_info skl_hw_codec_info = {
+    .dec_hw_context_init = gen9_dec_hw_context_init,
+    .enc_hw_context_init = gen9_enc_hw_context_init,
+    .proc_hw_context_init = gen75_proc_context_init,
+    .render_init = gen9_render_init,
+    .post_processing_context_init = gen9_post_processing_context_init,
+
+    .max_width = 4096,
+    .max_height = 4096,
+    .min_linear_wpitch = 64,
+    .min_linear_hpitch = 16,
+
+    .h264_mvc_dec_profiles = (VA_PROFILE_MASK(H264StereoHigh) |
+                              VA_PROFILE_MASK(H264MultiviewHigh)),
+    .h264_dec_chroma_formats = EXTRA_H264_DEC_CHROMA_FORMATS,
+    .jpeg_dec_chroma_formats = EXTRA_JPEG_DEC_CHROMA_FORMATS,
+    .jpeg_enc_chroma_formats = EXTRA_JPEG_ENC_CHROMA_FORMATS,
+
+    .has_mpeg2_decoding = 1,
+    .has_mpeg2_encoding = 1,
+    .has_h264_decoding = 1,
+    .has_h264_encoding = 1,
+    .has_vc1_decoding = 1,
+    .has_jpeg_decoding = 1,
+    .has_jpeg_encoding = 1,
+    .has_vpp = 1,
+    .has_accelerated_getimage = 1,
+    .has_accelerated_putimage = 1,
+    .has_tiled_surface = 1,
+    .has_di_motion_adptive = 1,
+    .has_di_motion_compensated = 1,
+    .has_vp8_decoding = 1,
+    .has_h264_mvc_encoding = 1,
+    .has_hevc_decoding = 1,
+
+    .num_filters = 5,
+    .filters = {
+        { VAProcFilterNoiseReduction, I965_RING_VEBOX },
+        { VAProcFilterDeinterlacing, I965_RING_VEBOX },
+        { VAProcFilterSharpening, I965_RING_NULL }, /* need to rebuild the shader for BDW */
+        { VAProcFilterColorBalance, I965_RING_VEBOX},
+        { VAProcFilterSkinToneEnhancement, I965_RING_VEBOX},
+    },
+};
+
 struct hw_codec_info *
 i965_get_codec_info(int devid)
 {
@@ -415,6 +475,13 @@ static const struct intel_device_info chv_device_info = {
     .is_cherryview = 1,
 };
 
+static const struct intel_device_info skl_device_info = {
+    .gen = 9,
+
+    .urb_size = 4096,
+    .max_wm_threads = 64,       /* per PSD */
+};
+
 const struct intel_device_info *
 i965_get_device_info(int devid)
 {
@@ -438,7 +505,6 @@ static void cpuid(unsigned int op,
  * This function doesn't check the length. And the caller should
  * assure that the length of input string should be greater than 48.
  */
-
 static int intel_driver_detect_cpustring(char *model_id)
 {
     uint32_t *rdata;
@@ -493,18 +559,18 @@ static void hsw_hw_codec_preinit(VADriverContextP ctx, struct hw_codec_info *cod
     /* strip the cpufreq info */
     model_ptr = model_string;
     tmp_ptr = strstr(model_ptr, "@");
-
+   
     if (tmp_ptr)
         *tmp_ptr = '\0';
 
-    /* strip the space character */
+    /* strip the space character and convert to the lower case */
     model_ptr = model_string;
     model_len = strlen(model_string);
     for (i = 0; i < model_len; i++) {
-        if (model_string[i] != ' ') {
-            *model_ptr = model_string[i];
-            model_ptr++;
-        }
+         if (model_string[i] != ' ') {
+             *model_ptr = model_string[i];
+             model_ptr++;
+         }
     }
     *model_ptr = '\0';
 
@@ -528,4 +594,127 @@ static void hsw_hw_codec_preinit(VADriverContextP ctx, struct hw_codec_info *cod
 	codec_info->has_h264_mvc_encoding = 0;
 	codec_info->has_mpeg2_encoding = 0;
     }
+    return;
+}
+
+/*
+ * the hook_list for Sandybride.
+ * It is captured by /proc/cpuinfo and the space character is stripped.
+ */
+const static char *gen6_cpu_hook_list[] =  {
+"Intel(R)Celeron(R)CPU847",
+"Intel(R)Celeron(R)CPU867",
+};
+
+static void gen6_hw_codec_preinit(VADriverContextP ctx, struct hw_codec_info *codec_info)
+{
+    char model_string[64];
+    char *model_ptr, *tmp_ptr;
+    int i, model_len, list_len;
+    bool found;
+
+    memset(model_string, 0, sizeof(model_string));
+
+    /* If it can't detect cpu model_string, leave it alone */
+    if (intel_driver_detect_cpustring(model_string))
+        return;
+
+    /* strip the cpufreq info */
+    model_ptr = model_string;
+    tmp_ptr = strstr(model_ptr, "@");
+
+    if (tmp_ptr)
+        *tmp_ptr = '\0';
+
+    /* strip the space character and convert to the lower case */
+    model_ptr = model_string;
+    model_len = strlen(model_string);
+    for (i = 0; i < model_len; i++) {
+         if (model_string[i] != ' ') {
+             *model_ptr = model_string[i];
+             model_ptr++;
+         }
+    }
+    *model_ptr = '\0';
+
+    found = false;
+    list_len = sizeof(gen6_cpu_hook_list) / sizeof(char *);
+    model_len = strlen(model_string);
+    for (i = 0; i < list_len; i++) {
+        model_ptr = (char *)gen6_cpu_hook_list[i];
+
+        if (strlen(model_ptr) != model_len)
+            continue;
+
+        if (strncasecmp(model_string, model_ptr, model_len) == 0) {
+            found = true;
+            break;
+        }
+    }
+
+    if (found) {
+        codec_info->has_h264_encoding = 0;
+    }
+    return;
+}
+
+/*
+ * the hook_list for Ivybridge.
+ * It is captured by /proc/cpuinfo and the space character is stripped.
+ */
+const static char *gen7_cpu_hook_list[] =  {
+"Intel(R)Celeron(R)CPU1007U",
+};
+
+static void gen7_hw_codec_preinit(VADriverContextP ctx, struct hw_codec_info *codec_info)
+{
+    char model_string[64];
+    char *model_ptr, *tmp_ptr;
+    int i, model_len, list_len;
+    bool found;
+
+    memset(model_string, 0, sizeof(model_string));
+
+    /* If it can't detect cpu model_string, leave it alone */
+    if (intel_driver_detect_cpustring(model_string))
+        return;
+
+    /* strip the cpufreq info */
+    model_ptr = model_string;
+    tmp_ptr = strstr(model_ptr, "@");
+
+    if (tmp_ptr)
+        *tmp_ptr = '\0';
+
+    /* strip the space character and convert to the lower case */
+    model_ptr = model_string;
+    model_len = strlen(model_string);
+    for (i = 0; i < model_len; i++) {
+         if (model_string[i] != ' ') {
+             *model_ptr = model_string[i];
+             model_ptr++;
+         }
+    }
+    *model_ptr = '\0';
+
+    found = false;
+    list_len = sizeof(gen7_cpu_hook_list) / sizeof(char *);
+    model_len = strlen(model_string);
+    for (i = 0; i < list_len; i++) {
+        model_ptr = (char *)gen7_cpu_hook_list[i];
+
+        if (strlen(model_ptr) != model_len)
+            continue;
+
+        if (strncasecmp(model_string, model_ptr, model_len) == 0) {
+            found = true;
+            break;
+	}
+    }
+
+    if (found) {
+        codec_info->has_h264_encoding = 0;
+        codec_info->has_mpeg2_encoding = 0;
+    }
+    return;
 }
