@@ -122,6 +122,131 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
     return VA_STATUS_SUCCESS;
 }
 
+
+static VAStatus
+intel_encoder_check_jpeg_yuv_surface(VADriverContextP ctx,
+                                VAProfile profile,
+                                struct encode_state *encode_state,
+                                struct intel_encoder_context *encoder_context)
+{
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    struct i965_surface src_surface, dst_surface;
+    struct object_surface *obj_surface;
+    VAStatus status;
+    VARectangle rect;
+    int format=0, fourcc=0, subsample=0;
+
+    /* releae the temporary surface */
+    if (encoder_context->is_tmp_id) {
+        i965_DestroySurfaces(ctx, &encoder_context->input_yuv_surface, 1);
+        encode_state->input_yuv_object = NULL;
+    }
+
+    encoder_context->is_tmp_id = 0;
+    obj_surface = SURFACE(encode_state->current_render_target);
+    assert(obj_surface && obj_surface->bo);
+
+    if (!obj_surface || !obj_surface->bo)
+        return VA_STATUS_ERROR_INVALID_PARAMETER;
+
+    unsigned int tiling = 0, swizzle = 0;
+
+    dri_bo_get_tiling(obj_surface->bo, &tiling, &swizzle);
+
+    if (tiling == I915_TILING_Y) {
+        if( (obj_surface->fourcc==VA_FOURCC_NV12)  || (obj_surface->fourcc==VA_FOURCC_UYVY) ||
+            (obj_surface->fourcc==VA_FOURCC_YUY2)  || (obj_surface->fourcc==VA_FOURCC_Y800) ||
+            (obj_surface->fourcc==VA_FOURCC_RGBA)  || (obj_surface->fourcc==VA_FOURCC_444P) ) {
+            encoder_context->input_yuv_surface = encode_state->current_render_target;
+            encode_state->input_yuv_object = obj_surface;
+            return VA_STATUS_SUCCESS;
+        }
+    }
+
+    rect.x = 0;
+    rect.y = 0;
+    rect.width = obj_surface->orig_width;
+    rect.height = obj_surface->orig_height;
+
+    src_surface.base = (struct object_base *)obj_surface;
+    src_surface.type = I965_SURFACE_TYPE_SURFACE;
+    src_surface.flags = I965_SURFACE_FLAG_FRAME;
+
+    switch( obj_surface->fourcc) {
+
+        case VA_FOURCC_YUY2:
+            fourcc = VA_FOURCC_YUY2;
+            format = VA_RT_FORMAT_YUV422;
+            subsample = SUBSAMPLE_YUV422H;
+            break;
+
+        case VA_FOURCC_UYVY:
+            fourcc = VA_FOURCC_UYVY;
+            format = VA_RT_FORMAT_YUV422;
+            subsample = SUBSAMPLE_YUV422H;
+            break;
+
+        case VA_FOURCC_Y800:
+            fourcc = VA_FOURCC_Y800;
+            format = VA_RT_FORMAT_YUV400;
+            subsample = SUBSAMPLE_YUV400;
+            break;
+
+        case VA_FOURCC_444P:
+            fourcc = VA_FOURCC_444P;
+            format = VA_RT_FORMAT_YUV444;
+            subsample = SUBSAMPLE_YUV444;
+            break;
+
+        case VA_FOURCC_RGBA:
+            fourcc = VA_FOURCC_RGBA;
+            format = VA_RT_FORMAT_RGB32;
+            subsample = SUBSAMPLE_RGBX;
+            break;
+
+        default: //All other scenarios will have NV12 format
+            fourcc = VA_FOURCC_NV12;
+            format = VA_RT_FORMAT_YUV420;
+            subsample = SUBSAMPLE_YUV420;
+            break;
+    }
+
+    status = i965_CreateSurfaces(ctx,
+                                 obj_surface->orig_width,
+                                 obj_surface->orig_height,
+                                 format,
+                                 1,
+                                 &encoder_context->input_yuv_surface);
+    assert(status == VA_STATUS_SUCCESS);
+
+    if (status != VA_STATUS_SUCCESS)
+        return status;
+
+    obj_surface = SURFACE(encoder_context->input_yuv_surface);
+    encode_state->input_yuv_object = obj_surface;
+    assert(obj_surface);
+    i965_check_alloc_surface_bo(ctx, obj_surface, 1, fourcc, subsample);
+
+    dst_surface.base = (struct object_base *)obj_surface;
+    dst_surface.type = I965_SURFACE_TYPE_SURFACE;
+    dst_surface.flags = I965_SURFACE_FLAG_FRAME;
+
+    //The Y800 format is expected to be tiled.
+    //Linear Y800 is a corner case and needs code in the i965_image_processing.
+    if(obj_surface->fourcc != VA_FOURCC_Y800){
+        status = i965_image_processing(ctx,
+                                   &src_surface,
+                                   &rect,
+                                   &dst_surface,
+                                   &rect);
+        assert(status == VA_STATUS_SUCCESS);
+    }
+
+    encoder_context->is_tmp_id = 1;
+
+    return VA_STATUS_SUCCESS;
+}
+
 static VAStatus
 intel_encoder_check_misc_parameter(VADriverContextP ctx,
                                   struct encode_state *encode_state,
@@ -270,6 +395,32 @@ error:
     return VA_STATUS_ERROR_INVALID_PARAMETER;
 }
 
+static VAStatus
+intel_encoder_check_jpeg_parameter(VADriverContextP ctx,
+                                  struct encode_state *encode_state,
+                                  struct intel_encoder_context *encoder_context)
+{
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    struct object_buffer *obj_buffer;
+    VAEncPictureParameterBufferJPEG *pic_param = (VAEncPictureParameterBufferJPEG *)encode_state->pic_param_ext->buffer;
+
+
+    assert(!(pic_param->pic_flags.bits.profile)); //Baseline profile is 0.
+
+    obj_buffer = BUFFER(pic_param->coded_buf);
+    assert(obj_buffer && obj_buffer->buffer_store && obj_buffer->buffer_store->bo);
+
+    if (!obj_buffer || !obj_buffer->buffer_store || !obj_buffer->buffer_store->bo)
+        goto error;
+
+    encode_state->coded_buf_object = obj_buffer;
+
+    return VA_STATUS_SUCCESS;
+
+error:
+    return VA_STATUS_ERROR_INVALID_PARAMETER;
+}
+
 static VAStatus
 intel_encoder_sanity_check_input(VADriverContextP ctx,
                                  VAProfile profile,
@@ -283,25 +434,36 @@ intel_encoder_sanity_check_input(VADriverContextP ctx,
     case VAProfileH264Main:
     case VAProfileH264High:
     case VAProfileH264MultiviewHigh:
-    case VAProfileH264StereoHigh:
+    case VAProfileH264StereoHigh: {
         vaStatus = intel_encoder_check_avc_parameter(ctx, encode_state, encoder_context);
+        if (vaStatus != VA_STATUS_SUCCESS)
+            goto out;
+        vaStatus = intel_encoder_check_yuv_surface(ctx, profile, encode_state, encoder_context);
         break;
+    }
 
     case VAProfileMPEG2Simple:
-    case VAProfileMPEG2Main:
+    case VAProfileMPEG2Main: {
         vaStatus = intel_encoder_check_mpeg2_parameter(ctx, encode_state, encoder_context);
+        if (vaStatus != VA_STATUS_SUCCESS)
+            goto out;
+        vaStatus = intel_encoder_check_yuv_surface(ctx, profile, encode_state, encoder_context);
         break;
+    }
+
+    case VAProfileJPEGBaseline:  {
+        vaStatus = intel_encoder_check_jpeg_parameter(ctx, encode_state, encoder_context);
+        if (vaStatus != VA_STATUS_SUCCESS)
+            goto out;
+        vaStatus = intel_encoder_check_jpeg_yuv_surface(ctx, profile, encode_state, encoder_context);
+        break;
+    }
 
     default:
         vaStatus = VA_STATUS_ERROR_UNSUPPORTED_PROFILE;
         break;
     }
 
-    if (vaStatus != VA_STATUS_SUCCESS)
-        goto out;
-
-    vaStatus = intel_encoder_check_yuv_surface(ctx, profile, encode_state, encoder_context);
-
     if (vaStatus == VA_STATUS_SUCCESS)
         vaStatus = intel_encoder_check_misc_parameter(ctx, encode_state, encoder_context);
 
@@ -326,7 +488,9 @@ intel_encoder_end_picture(VADriverContextP ctx,
 
     encoder_context->mfc_brc_prepare(encode_state, encoder_context);
 
-    vaStatus = encoder_context->vme_pipeline(ctx, profile, encode_state, encoder_context);
+    if((encoder_context->vme_context && encoder_context->vme_pipeline)) {
+        vaStatus = encoder_context->vme_pipeline(ctx, profile, encode_state, encoder_context);
+    }
 
     if (vaStatus == VA_STATUS_SUCCESS)
         encoder_context->mfc_pipeline(ctx, profile, encode_state, encoder_context);
@@ -339,7 +503,10 @@ intel_encoder_context_destroy(void *hw_context)
     struct intel_encoder_context *encoder_context = (struct intel_encoder_context *)hw_context;
 
     encoder_context->mfc_context_destroy(encoder_context->mfc_context);
-    encoder_context->vme_context_destroy(encoder_context->vme_context);
+
+    if (encoder_context->vme_context_destroy && encoder_context->vme_context)
+       encoder_context->vme_context_destroy(encoder_context->vme_context);
+
     intel_batchbuffer_free(encoder_context->base.batch);
     free(encoder_context);
 }
@@ -382,6 +549,10 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     case VAProfileH264MultiviewHigh:
         encoder_context->codec = CODEC_H264_MVC;
         break;
+        
+    case VAProfileJPEGBaseline:
+        encoder_context->codec = CODEC_JPEG;
+        break;
 
     default:
         /* Never get here */
@@ -404,9 +575,11 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     }
 
     vme_context_init(ctx, encoder_context);
-    assert(encoder_context->vme_context);
-    assert(encoder_context->vme_context_destroy);
-    assert(encoder_context->vme_pipeline);
+    if(obj_config->profile != VAProfileJPEGBaseline) {
+        assert(encoder_context->vme_context);
+        assert(encoder_context->vme_context_destroy);
+        assert(encoder_context->vme_pipeline);
+    }
 
     mfc_context_init(ctx, encoder_context);
     assert(encoder_context->mfc_context);
@@ -441,3 +614,11 @@ gen8_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config)
     return intel_enc_hw_context_init(ctx, obj_config, gen8_vme_context_init, gen8_mfc_context_init);
 }
 
+struct hw_context *
+gen9_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config)
+{
+    if (obj_config->profile == VAProfileJPEGBaseline)
+        return intel_enc_hw_context_init(ctx, obj_config, gen8_vme_context_init, gen8_mfc_context_init);
+    else
+        return intel_enc_hw_context_init(ctx, obj_config, gen9_vme_context_init, gen9_mfc_context_init);
+}
